@@ -457,6 +457,7 @@ MUIMSessionClose(MUIMSessionHandle inSessionHandle)
     (*inSessionHandle)->fSegments = NULL;
     (*inSessionHandle)->fSegmentCount = 0;
 
+    uim_release_context((*inSessionHandle)->fUC);
     DisposeHandle((Handle) inSessionHandle);
   }
 }
@@ -674,14 +675,16 @@ ComponentResult
 MUIMSetInputMode(MUIMSessionHandle inSessionHandle, CFStringRef inInputMode)
 {
   ComponentResult result;
+  char modeName[BUFSIZ];
   
   if (!inSessionHandle || !(*inSessionHandle) ||
       !(*inSessionHandle)->fUC || !inInputMode)
     return tsmInputModeChangeFailedErr;
   
+  CFStringGetCString(inInputMode, modeName, BUFSIZ, kCFStringEncodingMacRoman);
+  
 #if DEBUG_INPUTMODE
-  DEBUG_PRINT("MUIMSetInputMode() inInputModer='%s'\n",
-              CFStringGetCStringPtr(inInputMode, kCFStringEncodingMacRoman));
+  DEBUG_PRINT("MUIMSetInputMode() inInputModer='%s'\n", modeName);
 #endif
   
   if (gMacUIMDisable)
@@ -693,8 +696,6 @@ MUIMSetInputMode(MUIMSessionHandle inSessionHandle, CFStringRef inInputMode)
   if (!(*inSessionHandle)->fLastMode ||
       CFStringCompare(inInputMode, (*inSessionHandle)->fLastMode, 0)) {
     
-    //DEBUG_PRINT("MUIMSetInputMode() differenct mode\n");
-  
     if (!CFStringCompare(inInputMode,
                          kTextServiceInputModeRoman, 0) ||
         !CFStringCompare(inInputMode,
@@ -702,12 +703,16 @@ MUIMSetInputMode(MUIMSessionHandle inSessionHandle, CFStringRef inInputMode)
 #if 0
       UInt32 i;
       for (i = 0; i < gNumSession; i++) {
+        BlockUpdatePreedit();
         uim_press_key((*gSessionList[i])->fUC, UKey_Private1, 0);
         uim_release_key((*gSessionList[i])->fUC, UKey_Private1, 0);
+        UnblockUpdatePreedit();
       }
 #else
+      BlockUpdatePreedit();
       uim_press_key((*inSessionHandle)->fUC, UKey_Private1, 0);
       uim_release_key((*inSessionHandle)->fUC, UKey_Private1, 0);
+      UnblockUpdatePreedit();
 #endif
     }
     else if (!CFStringCompare(inInputMode,
@@ -729,12 +734,16 @@ MUIMSetInputMode(MUIMSessionHandle inSessionHandle, CFStringRef inInputMode)
 #if 0
       UInt32 i;
       for (i = 0; i < gNumSession; i++) {
+        BlockUpdatePreedit();
         uim_press_key((*gSessionList[i])->fUC, UKey_Private2, 0);
         uim_release_key((*gSessionList[i])->fUC, UKey_Private2, 0);
+        UnblockUpdatePreedit();
       }
 #else
+      BlockUpdatePreedit();
       uim_press_key((*inSessionHandle)->fUC, UKey_Private2, 0);
       uim_release_key((*inSessionHandle)->fUC, UKey_Private2, 0);
+      UnblockUpdatePreedit();
 #endif
     }
 
@@ -930,23 +939,21 @@ MUIMPencilMenuEventHandler(EventHandlerCallRef inEventHandlerCallRef,
 static void
 CreateUIMContext(MUIMSessionHandle inSessionHandle, CFStringRef inIMName)
 {
-  const char *imName;
+  char imName[BUFSIZ];
   CFPropertyListRef imVal = NULL;
 
-  if (inIMName) {
-    imName = CFStringGetCStringPtr(inIMName, kCFStringEncodingMacRoman);
-  }
-  else if (gIMName) {
-    imName = CFStringGetCStringPtr(gIMName, kCFStringEncodingMacRoman);
-  }
+  if (inIMName)
+    CFStringGetCString(inIMName, imName, BUFSIZ, kCFStringEncodingMacRoman);
+  else if (gIMName)
+    CFStringGetCString(gIMName, imName, BUFSIZ, kCFStringEncodingMacRoman);
   else {
     // Load the preferences
     imVal = CFPreferencesCopyAppValue(CFSTR(kPrefIM), CFSTR(kAppID));
     if (imVal && CFGetTypeID(imVal) == CFStringGetTypeID())
-      imName = CFStringGetCStringPtr((CFStringRef) imVal,
-                                     kCFStringEncodingMacRoman);
+      CFStringGetCString((CFStringRef) imVal, imName, BUFSIZ,
+                         kCFStringEncodingMacRoman);
     else
-      imName = kDefaultIM;
+      strcpy(imName, kDefaultIM);
   }
 
   (*inSessionHandle)->fUC =
@@ -1036,6 +1043,7 @@ NotificationCallback(CFNotificationCenterRef inCenter, void *inObserver,
                      CFDictionaryRef inUserInfo)
 {
   CFStringRef im;
+  char imName[BUFSIZ];
   CFBooleanRef on;
   CFStringRef fontName;
   CFNumberRef fontSize;
@@ -1043,10 +1051,10 @@ NotificationCallback(CFNotificationCenterRef inCenter, void *inObserver,
   UInt32 i;
   
   im = CFDictionaryGetValue(inUserInfo, CFSTR(kPrefIM));
+  CFStringGetCString(im, imName, BUFSIZ, kCFStringEncodingMacRoman);
 
 #if DEBUG_NOTIFY
-  DEGUG_PRINT("NotificationCallback() im='%s'\n",
-              CFStringGetCStringPtr(im, kCFStringEncodingMacRoman));
+  DEGUG_PRINT("NotificationCallback() im='%s'\n", imName);
 #endif
   
   if ((on = CFDictionaryGetValue(inUserInfo, CFSTR(kPrefCandVertical))))
@@ -1060,12 +1068,6 @@ NotificationCallback(CFNotificationCenterRef inCenter, void *inObserver,
     if ((fontSize = CFDictionaryGetValue(inUserInfo, CFSTR(kPrefCandFontSize))))
       CFNumberGetValue(fontSize, kCFNumberFloatType, &gCandFontSize);
 
-#if DEBUG_NOTIFY
-    DEBUG_PRINT("NotificationCallback() fontName=%s fontSize=%f\n",
-                CFStringGetCStringPtr(gCandFont, kCFStringEncodingMacRoman),
-                gCandFontSize);
-#endif
-  
     for (i = 0; i < gNumSession; i++)
       SetCandidateFont(gSessionList[i], gCandFont, gCandFontSize);
   }
