@@ -196,4 +196,10 @@ GetPreeditString(MUIMSessionHandle inHandle, UniCharPtr *outStr,
 void
 DumpString(const char *name, const char *str, int len);
 
+void
+BlockUpdatePreedit();
+
+void
+UnblockUpdatePreedit();
+
 #endif // MUIM_h
