@@ -66,14 +66,14 @@ convertHelperString(char *str);
   sharedExtra = self;
 
   menu = [[NSMenu alloc] initWithTitle:@""];
-  //[menu setAutoenablesItems:NO];
 
   view = [[MacUIMHelperView alloc] initWithFrame:[[self view] frame]
                                    menuExtra:self];
   [self setView:view];
 
-  //uim_helper_client_get_prop_list();
-
+#ifndef NEW_HELPER
+  uimFD = -1;
+#endif
   [self helperConnect];
 
   imName = nil;
@@ -116,8 +116,6 @@ convertHelperString(char *str);
 
   [self helperDisconnect];
 
-  //[uimHandle release];
-
   [modes release];
   [propNames release];
   [menuItems release];
@@ -152,11 +150,16 @@ convertHelperString(char *str);
     image = [[[NSImage alloc] initWithSize:NSMakeSize(kMenuBarWidth, kMenuBarHeight)]
               autorelease];
   }
-  else {
+  else if ([labels count] == 2) {
     [view setFrameSize:NSMakeSize(kMenuBarWidth2, [view frame].size.height)];
     [self setLength:kMenuBarWidth2];
     image = [[[NSImage alloc] initWithSize:NSMakeSize(kMenuBarWidth2, kMenuBarHeight)]
               autorelease];
+  } else {
+    [view setFrameSize:NSMakeSize(kMenuBarWidth3, [view frame].size.height)];
+    [self setLength:kMenuBarWidth3];
+    image = [[[NSImage alloc] initWithSize:NSMakeSize(kMenuBarWidth3, kMenuBarHeight)]
+              autorelease];
   }
 
   [self renderFrame:image];
@@ -190,7 +193,7 @@ convertHelperString(char *str);
     [strokeColor set];
     [framePath stroke];
   }
-  else {
+  else if ([labels count] == 2) {
     framePath =
       [NSBezierPath bezierPathWithRect:NSMakeRect(0.5, 3.5,
                                                   kMenuBarWidth2 - 1.0, kMenuBarHeight - 7.0)];
@@ -207,6 +210,31 @@ convertHelperString(char *str);
     [strokeColor set];
     [framePath stroke];
   }
+  else {
+    framePath =
+      [NSBezierPath bezierPathWithRect:NSMakeRect(0.5, 3.5,
+                                                  kMenuBarWidth3 - 1.0, kMenuBarHeight - 7.0)];
+    [fillColor set];
+    [framePath fill];
+    [strokeColor set];
+    [framePath stroke];
+
+    framePath =
+      [NSBezierPath bezierPathWithRect:NSMakeRect(kMenuBarWidth3 / 3.0, 3.5,
+                                                  0.1, kMenuBarHeight - 7.0)];
+    [fillColor set];
+    [framePath fill];
+    [strokeColor set];
+    [framePath stroke];
+
+    framePath =
+      [NSBezierPath bezierPathWithRect:NSMakeRect(kMenuBarWidth3 * 2.0 / 3.0, 3.5,
+                                                  0.1, kMenuBarHeight - 7.0)];
+    [fillColor set];
+    [framePath fill];
+    [strokeColor set];
+    [framePath stroke];
+  }
 
   [image unlockFocus];
 }
@@ -235,13 +263,7 @@ convertHelperString(char *str);
     if (!label)
       label = [[NSMutableString alloc] initWithString:[labels objectAtIndex:i]];
 
-    //[label appendString:@" "];
-
     if ([labels count] <= 1) {
-      // aString = [[[NSAttributedString alloc] initWithString:@"string" attributes:[self boldFontAttributes]] autorelease];
-      // NSDictionary dictionaryWithObject: [NSFont boldSystemFontOfSize:[NSFont systemFontSize]]
-
-
       text = [[NSAttributedString alloc] initWithString:label
                                          attributes:[NSDictionary dictionaryWithObjectsAndKeys:
                                                                     [NSFont boldSystemFontOfSize:12],
@@ -254,7 +276,7 @@ convertHelperString(char *str);
                                      ceil([text size].width + 0.5)) / 2.0,
                                     (kMenuBarHeight - ceil([text size].height)) / 2.0)];
     }
-    else {
+    else if ([labels count] == 2) {
       text = [[NSAttributedString alloc] initWithString:label
                                          attributes:[NSDictionary dictionaryWithObjectsAndKeys:
                                                                     [NSFont boldSystemFontOfSize:11],
@@ -268,6 +290,38 @@ convertHelperString(char *str);
                                      ceil([text size].width + 0.5)) / 2.0,
                                     (kMenuBarHeight - ceil([text size].height)) / 2.0)];
     }
+    else {
+      text = [[NSAttributedString alloc] initWithString:label
+                                         attributes:[NSDictionary dictionaryWithObjectsAndKeys:
+                                                                    [NSFont boldSystemFontOfSize:11],
+                                                                  NSFontAttributeName,
+                                                                  color,
+                                                                  NSForegroundColorAttributeName,
+                                                                  nil]];
+      if (i == 0) {
+        [text drawAtPoint:NSMakePoint(0.5 + 0.5 +
+                                    0 + (kMenuBarWidth3 / 3.0 - 0.5 -
+                                     ceil([text size].width + 0.5)) / 2.0,
+                                    (kMenuBarHeight - ceil([text size].height)) / 2.0)];
+
+      }
+      else if (i == 1) {
+        [text drawAtPoint:NSMakePoint(0.5 + 0.5 +
+                                    kMenuBarWidth3 / 3.0
+                                    + (kMenuBarWidth3 / 3.0 - 0.5 -
+                                     ceil([text size].width + 0.5)) / 2.0,
+                                    (kMenuBarHeight - ceil([text size].height)) / 2.0)];
+
+      }
+      else if (i == 2) {
+        [text drawAtPoint:NSMakePoint(0.5 + 0.5 +
+                                    kMenuBarWidth3 * 2.0 / 3.0
+                                    + (kMenuBarWidth3 / 3.0 - 0.5 -
+                                     ceil([text size].width + 0.5)) / 2.0,
+                                    (kMenuBarHeight - ceil([text size].height)) / 2.0)];
+
+      }
+    }
 
     [text release];
     [label release];
@@ -280,8 +334,6 @@ convertHelperString(char *str);
 
 - (NSMenu *)menu
 {
-  //[self updateMenu];
-
   return menu;
 }
 
@@ -398,6 +450,11 @@ convertHelperString(char *str);
 
 - (int)helperConnect
 {
+#ifndef NEW_HELPER
+  if (uimFD >= 0)
+    return uimFD;
+
+#endif
   NSNotificationCenter *noc = [NSNotificationCenter defaultCenter];
 
 #ifdef NEW_HELPER
@@ -461,8 +518,6 @@ convertHelperString(char *str);
     [self helperDisconnect];
     [self helperConnect];
   }
-  
-  //[self loadPrefs];
 }
 
 - (void)helperParse:(char *)str
@@ -496,13 +551,11 @@ convertHelperString(char *str);
                         options:NSCaseInsensitiveSearch
                         range:NSMakeRange(0, strlen("focus_in"))]
                  == NSOrderedSame) {
-          //[view setNeedsDisplay:YES];
         }
         else if ([first compare:@"focus_out"
                         options:NSCaseInsensitiveSearch
                         range:NSMakeRange(0, strlen("focus_out"))]
                  == NSOrderedSame) {
-          //[labels removeAllObjects];
         }
 #if DEBUG_HELPER_EXTRA
         else {
@@ -554,8 +607,6 @@ convertHelperString(char *str);
   if (!lines || [lines count] < 2)
     return;
 
-  line = [lines objectAtIndex:1];
-
   [labels removeAllObjects];
   [modes removeAllObjects];
   [propNames removeAllObjects];
@@ -571,16 +622,27 @@ convertHelperString(char *str);
       col = [cols objectAtIndex:0];
       if ([col compare:@"branch"] == NSOrderedSame) {
         NSMutableString *branch =
-          [[NSMutableString alloc] initWithString:[cols objectAtIndex:1]];
+          [[NSMutableString alloc] initWithString:[cols objectAtIndex:2]];
         [labels addObject:branch];
       }
       else if ([col compare:@"leaf"] == NSOrderedSame) {
         NSMutableString *mode =
-          [[NSMutableString alloc] initWithString:[cols objectAtIndex:2]];
+          [[NSMutableString alloc] initWithString:[cols objectAtIndex:3]];
         NSMutableString *prop =
-          [[NSMutableString alloc] initWithString:[cols objectAtIndex:4]];
-        [modes addObject:mode];
-        [propNames addObject:prop];
+          [[NSMutableString alloc] initWithString:[cols objectAtIndex:5]];
+        
+        // disable uim's IM-switch
+        if ([prop compare:@"action_imsw_"
+                   options:nil
+                     range:NSMakeRange(0, strlen("action_imsw_"))]
+            == NSOrderedSame) {
+          [mode release];
+          [prop release];
+        }
+        else {
+          [modes addObject:mode];
+          [propNames addObject:prop];
+        }
       }
     }
   }
@@ -607,8 +669,6 @@ convertHelperString(char *str);
 
   charset = [cols objectAtIndex:1];
 
-  //uim_helper_client_get_prop_list();
-
   [labels removeAllObjects];
 
   for (i = 2; i < [lines count]; i++) {
