@@ -356,9 +356,15 @@ done:
 
 - (IBAction)chooseFont:(id)sender
 {
+  [[fontSample window] setDelegate:self];
+  [[fontSample window] makeFirstResponder:[fontSample window]];
   [[NSFontManager sharedFontManager] orderFrontFontPanel:self];
-  [[NSFontManager sharedFontManager] setSelectedFont:font
-                                          isMultiple:NO];
+  [[NSFontManager sharedFontManager] setSelectedFont:font isMultiple:NO];
+}
+
+- (void)changeFont:(id)fontManager
+{
+  [self setFont:(id) self];
 }
 
 - (void)setFont:(id)sender
