@@ -61,6 +61,10 @@ extern Boolean gActiveProp;
 extern Boolean gEnableModeTips;
 extern Boolean gCandVertical;
 
+static Boolean gBlockUpdatePreedit = false;
+
+#pragma mark -
+
 static void
 AddPreeditSegment(MUIMSessionHandle inHandle, int inAttr,
                   const char *inStr);
@@ -160,12 +164,6 @@ UIMPreeditPushback(void *ptr, int attr, const char *str)
   }
 #endif
 
-  /*
-  if (!strcmp(str, "")
-      && !(attr & (UPreeditAttr_Cursor | UPreeditAttr_Separator | UPreeditAttr_UnderLine)))
-    return;
-  */
-
   AddPreeditSegment(handle, attr, str);
 }
 
@@ -181,7 +179,9 @@ UIMPreeditUpdate(void *ptr)
   DEBUG_PRINT("UIMPreeditUpdate()\n");
 #endif
 
-  MUIMUpdateActiveInputArea(handle, FALSE);
+  // Avoid a crash when Firefox is launched with uim(Japanese)
+  if (!gBlockUpdatePreedit)
+    MUIMUpdateActiveInputArea(handle, false);
 }
 
 /**
@@ -264,7 +264,8 @@ UIMCandShiftPage(void *inPtr, int inForward)
       // if the previous page is not exist
       // move to buttom page and select last candidate
       (*handle)->fCandidateIndex = (*handle)->fNRCandidates - 1;
-      (*handle)->fPageIndex = (*handle)->fNRCandidates / (*handle)->fDisplayLimit
+      (*handle)->fPageIndex = (*handle)->fNRCandidates
+        / (*handle)->fDisplayLimit
         + (((*handle)->fNRCandidates % (*handle)->fDisplayLimit) ? 1 : 0) - 1;
     }
     else {
@@ -301,6 +302,43 @@ UIMCandDeactivate(void *inPtr)
   (*handle)->fWindowOpened = false;
 }
 
+char *
+get_caret_state_label_from_prop_list(const char *str)
+{
+  const char *p, *q;
+  char *state_label = NULL;
+  char label[10];
+  int len, state_label_len = 0;
+
+  p = str;
+  while ((p = strstr(p, "branch\t"))) {
+    if ((p = strchr(p + 7, '\t'))) {
+      p++;
+      q = strchr(p, '\t');
+      len = q - p;
+      if (q && len < 10) {
+        strncpy(label, p, len);
+        label[len] = '\0';
+        if (!state_label) {
+          state_label_len = len;
+          state_label = strdup(label);
+        }
+        else {
+          state_label_len += (len + 1);
+          state_label = (char *) realloc(state_label, state_label_len + 1);
+          if (state_label) {
+            strcat(state_label, "\t");
+            strcat(state_label, label);
+            state_label[state_label_len] = '\0';
+          }
+        }
+      }
+    }
+  }
+
+  return state_label;
+}
+
 void
 UIMUpdatePropList(void *inPtr, const char *inStr)
 {
@@ -309,13 +347,8 @@ UIMUpdatePropList(void *inPtr, const char *inStr)
   char *tmp;
   
   if (!handle || !(*handle)) return;
-
-#if DEBUG_HELPER
-  //DEBUG_PRINT("UIMUpdatePropList() handle=%p inStr='%s'\n",
-  //            handle, inStr);
-#endif
-
-  if (!gActiveSession || !gPencilMenu) return;
+  if (!gActiveSession || !gPencilMenu || !gActiveProp) return;
+  if (gActiveSession != handle) return;
 
   tmp = (char *) malloc(sizeof(char) *
                         (strlen(kPropListUpdate) + 1 +
@@ -338,6 +371,25 @@ UIMUpdatePropList(void *inPtr, const char *inStr)
     }
     free(tmp);
   }
+
+  if (gEnableModeTips
+      && !(*handle)->fModeTipsBlock
+      ) {
+    char *label = get_caret_state_label_from_prop_list(inStr);;
+    CFStringRef allstr =
+      CFStringCreateWithCString(kCFAllocatorDefault,
+                                label, kCFStringEncodingUTF8);
+    free(label);
+    if (!allstr) return;
+    
+    CFArrayRef array =
+      CFStringCreateArrayBySeparatingStrings(kCFAllocatorDefault,
+                                             allstr, CFSTR("\t"));
+    if (array) {
+      ShowModeTips(handle, array);
+      CFRelease(array);
+    }
+  }
 }
 
 void
@@ -353,11 +405,12 @@ UIMUpdatePropLabel(void *inPtr, const char *inStr)
   DEBUG_PRINT("UIMUpdatePropLabel() time=%f handle=%p gActiveSession=%p\n",
               GetCurrentEventTime(),
               handle, gActiveSession);
-  //DEBUG_PRINT("UIMUpdatePropLabel() handle=%p gActiveSession=%p inStr='%s'\n",
-  //            handle, gActiveSession, inStr);
 #endif
 
-  if (!gActiveSession || !gPencilMenu) return;
+  if (!gActiveSession || !gPencilMenu || !gActiveProp) return;
+
+  if (gActiveSession != handle)
+    return;
 
   tmp = (char *) malloc(sizeof(char) *
                         (strlen(kPropLabelUpdate) + 1 +
@@ -380,13 +433,8 @@ UIMUpdatePropLabel(void *inPtr, const char *inStr)
     }
     free(tmp);
   }
-  
-  //DEBUG_PRINT("update label block=%s\n",
-  //            (*handle)->fModeTipsBlock ? "TRUE" : "FALSE");
 
-  if (gEnableModeTips
-      && !(*handle)->fModeTipsBlock
-      ) {
+  if (gEnableModeTips && !(*handle)->fModeTipsBlock) {
     CFStringRef allstr =
       CFStringCreateWithCString(kCFAllocatorDefault,
                                 inStr, kCFStringEncodingUTF8);
@@ -411,32 +459,36 @@ UIMCheckHelper()
       gUimHelperConnected = TRUE;
 
 #if DEBUG_HELPER
-      DEBUG_PRINT("UIMCheckHelper() gUimHelperConnected=%d\n", gUimHelperConnected);
+      DEBUG_PRINT("UIMCheckHelper() gUimHelperConnected=%d\n",
+                  gUimHelperConnected);
 #endif
 
       if (gUimHelperConnected) {
         if (!gUimSock) {
           gSockContext.version = 0;
-          //gSockContext.info = inHandle;
           gSockContext.info = NULL;
           gSockContext.retain = NULL;
           gSockContext.release = NULL;
           gSockContext.copyDescription = NULL;
           
-          gUimSock = CFSocketCreateWithNative(kCFAllocatorDefault, uim_helper_get_client_fd(),
-                                              kCFSocketReadCallBack, HelperRead,
+          gUimSock = CFSocketCreateWithNative(kCFAllocatorDefault,
+                                              uim_helper_get_client_fd(),
+                                              kCFSocketReadCallBack,
+                                              HelperRead,
                                               &gSockContext);
           if (!gUimSock) return;
         }
 
         if (!gUimRun) {
-          gUimRun = CFSocketCreateRunLoopSource(kCFAllocatorDefault, gUimSock, 0);
+          gUimRun = CFSocketCreateRunLoopSource(kCFAllocatorDefault,
+                                                gUimSock, 0);
           if (!gUimRun) {
             CFRelease(gUimSock);
             gUimSock = NULL;
             return;
           }
-          CFRunLoopAddSource(CFRunLoopGetCurrent(), gUimRun, kCFRunLoopDefaultMode);
+          CFRunLoopAddSource(CFRunLoopGetCurrent(), gUimRun,
+                             kCFRunLoopDefaultMode);
 #if DEBUG_HELPER
           DEBUG_PRINT("UIMCheckHelper() CFRunLoopGetCurrent()\n");
 #endif
@@ -457,7 +509,6 @@ UIMCheckHelper()
     if (gUimFD >= 0) {
       if (!gUimSock) {
         gSockContext.version = 0;
-        //gSockContext.info = inHandle;
         gSockContext.info = NULL;
         gSockContext.retain = NULL;
         gSockContext.release = NULL;
@@ -476,7 +527,8 @@ UIMCheckHelper()
           gUimSock = NULL;
           return;
         }
-        CFRunLoopAddSource(CFRunLoopGetCurrent(), gUimRun, kCFRunLoopDefaultMode);
+        CFRunLoopAddSource(CFRunLoopGetCurrent(), gUimRun,
+                           kCFRunLoopDefaultMode);
 #if DEBUG_HELPER
         DEBUG_PRINT("UIMCheckHelper() CFRunLoopGetCurrent()\n");
 #endif
@@ -515,7 +567,8 @@ ParseHelperString(const char *str)
   if (!gActiveSession) return;
 
   if (strncmp("im_change", str, 9) == 0) {
-    ParseIMChangeString(str);
+    if (gActiveProp)
+      ParseIMChangeString(str);
   }
   else if (strncmp("prop_list_get", str, 13) == 0) {
     if (gActiveProp)
@@ -525,7 +578,8 @@ ParseHelperString(const char *str)
     if (gActiveProp)
       uim_prop_label_update((*gActiveSession)->fUC);
   }
-  else if (strncmp("prop_activate", str, 13) == 0) {
+  else if (strncmp("prop_activate", str, 13) == 0 &&
+		  gActiveProp) {
     CFMutableStringRef cfstr;
     CFArrayRef array;
     CFStringRef first;
@@ -538,16 +592,14 @@ ParseHelperString(const char *str)
 
     if (array && CFArrayGetCount(array) > 0 &&
         (first = CFArrayGetValueAtIndex(array, 0))) {
-      CFStringRef second = CFArrayGetValueAtIndex(array, 1);
+      CFStringRef second = CFArrayGetValueAtIndex(array, 1);     
       if (second) {
-#if DEBUG_HELPER
-        DEBUG_PRINT("ParseHelperString() first='%s' second='%s'\n",
-                    CFStringGetCStringPtr(first, kCFStringEncodingUTF8),
-                    CFStringGetCStringPtr(second, kCFStringEncodingUTF8));
-#endif
+        char how[64];
+        CFStringGetCString(second, how, 64, kCFStringEncodingUTF8);
+        BlockUpdatePreedit();
         for (i = 0; i < gNumSession; i++)
-          uim_prop_activate((*gSessionList[i])->fUC,
-                            CFStringGetCStringPtr(second, kCFStringEncodingUTF8));
+          uim_prop_activate((*gSessionList[i])->fUC, how);
+        UnblockUpdatePreedit();
         CFRelease(second);
       }
       CFRelease(first);
@@ -558,35 +610,25 @@ ParseHelperString(const char *str)
   else if (strncmp("prop_label_update", str, 17) == 0) {
   }
   else if (strncmp("focus_in", str, 8) == 0) {
-    gActiveProp = FALSE;
-    //DEBUG_PRINT("deactivate prop\n");
+    gActiveProp = false;
   }
 }
 
 static void
 ParseIMChangeString(const char *str)
 {
-  CFStringRef allstr;
-  CFArrayRef array;
-  CFStringRef imName;
   UInt32 i;
+  char *eol, *im;
   
-  allstr = CFStringCreateWithCString(kCFAllocatorDefault,
-                                     str, kCFStringEncodingUTF8);
-  array = CFStringCreateArrayBySeparatingStrings(kCFAllocatorDefault,
-                                                 allstr, CFSTR("\n"));
-
-  imName = CFArrayGetValueAtIndex(array, 1);
-
-#if DEBUG_HELPER
-  DEBUG_PRINT("ParseIMChangeString() imName='%s'\n",
-              CFStringGetCStringPtr(imName, kCFStringEncodingUTF8));
-#endif
+  eol = strchr(str, '\n');
+  if (eol == NULL) return;
+  im = eol + 1;
+  eol = strchr(im, '\n');
+  if (eol == NULL) return;
+  *eol = '\0';
   
-  for (i = 0; i < gNumSession; i++) {
-    uim_switch_im((*gSessionList[i])->fUC,
-                   CFStringGetCStringPtr(imName, kCFStringEncodingUTF8));
-  }
+  for (i = 0; i < gNumSession; i++)
+    uim_switch_im((*gSessionList[i])->fUC, im);
 }
 
 void
@@ -605,7 +647,7 @@ UIMHelperDisconnect()
   gUimSock = NULL;
 
 #ifdef NEW_HELPER
-  gUimHelperConnected = FALSE;
+  gUimHelperConnected = false;
 #else
   gUimFD = -1;
 #endif
@@ -662,8 +704,6 @@ AddPreeditSegment(MUIMSessionHandle inHandle, int inAttr,
   unistr = (UniCharPtr) malloc(sizeof(UniChar) * (len + 1));
   CFStringGetCharacters(cf_string, CFRangeMake(0, len), unistr);
 
-  //DumpString("unistr", (char *) unistr, len * sizeof(UniChar));
-
   (*inHandle)->fSegments = realloc((*inHandle)->fSegments,
                                    sizeof(PreeditSegment) *
                                    ((*inHandle)->fSegmentCount + 1));
@@ -739,3 +779,15 @@ GetPreeditString(MUIMSessionHandle inHandle, UniCharPtr *outStr,
   else
     free(str);
 }
+
+void
+BlockUpdatePreedit()
+{
+  gBlockUpdatePreedit = true;
+}
+
+void
+UnblockUpdatePreedit()
+{
+  gBlockUpdatePreedit = false;
+}
